@@ -61,8 +61,8 @@ fn split(I: &mut [isize], V: &mut [isize], start: usize, len: usize, h: usize) {
                 }
                 i += 1;
             }
-            for i in 0..j {
-                V[usz(I[k + i])] = k as isize + j as isize - 1;
+            for &Ii in &I[k..k + j] {
+                V[usz(Ii)] = k as isize + j as isize - 1;
             }
             if j == 1 {
                 I[k] = -1;
@@ -73,11 +73,11 @@ fn split(I: &mut [isize], V: &mut [isize], start: usize, len: usize, h: usize) {
         let x = V[usz(I[start + len / 2] + h as isize)];
         let mut jj = 0;
         let mut kk = 0;
-        for i in start..start + len {
-            if V[usz(I[i] + h as isize)] < x {
+        for &Ii in &I[start..start + len] {
+            if V[usz(Ii + h as isize)] < x {
                 jj += 1;
             }
-            if V[usz(I[i] + h as isize)] == x {
+            if V[usz(Ii + h as isize)] == x {
                 kk += 1;
             }
         }
@@ -87,9 +87,10 @@ fn split(I: &mut [isize], V: &mut [isize], start: usize, len: usize, h: usize) {
         let mut k = 0;
         let mut i = start;
         while i < jj {
-            if V[usz(I[i] + h as isize)] < x {
+            let v = V[usz(I[i] + h as isize)];
+            if v < x {
                 i += 1;
-            } else if V[usz(I[i] + h as isize)] == x {
+            } else if v == x {
                 I.swap(i, jj + j);
                 j += 1;
             } else {
@@ -108,8 +109,8 @@ fn split(I: &mut [isize], V: &mut [isize], start: usize, len: usize, h: usize) {
         if jj > start {
             split(I, V, start, jj - start, h);
         }
-        for i in 0..kk - jj {
-            V[usz(I[jj + i])] = kk as isize - 1;
+        for &Ii in &I[jj..kk] {
+            V[usz(Ii)] = kk as isize - 1;
         }
         if jj == kk - 1 {
             I[jj] = -1;
@@ -122,25 +123,23 @@ fn split(I: &mut [isize], V: &mut [isize], start: usize, len: usize, h: usize) {
 
 fn qsufsort(I: &mut [isize], V: &mut [isize], old: &[u8]) {
     let mut buckets: [isize; 256] = [0; 256];
-    for i in 0..old.len() {
-        buckets[old[i] as usize] += 1;
+    for &o in old {
+        buckets[o as usize] += 1;
     }
     for i in 1..256 {
         buckets[i] += buckets[i - 1];
     }
-    let mut i = 255;
-    while i > 0 {
+    for i in (1..256).rev() {
         buckets[i] = buckets[i - 1];
-        i -= 1;
     }
     buckets[0] = 0;
-    for i in 0..old.len() {
-        buckets[old[i] as usize] += 1;
-        I[usz(buckets[old[i] as usize])] = i as isize;
+    for (i, old) in old.iter().copied().enumerate() {
+        buckets[old as usize] += 1;
+        I[usz(buckets[old as usize])] = i as isize;
     }
     I[0] = old.len() as isize;
-    for i in 0..old.len() {
-        V[i] = buckets[old[i] as usize];
+    for (i, old) in old.iter().copied().enumerate() {
+        V[i] = buckets[old as usize];
     }
     V[old.len()] = 0;
     for i in 1..256 {
@@ -170,10 +169,10 @@ fn qsufsort(I: &mut [isize], V: &mut [isize], old: &[u8]) {
         if len != 0 {
             I[usz(i - len)] = -len;
         }
-        h = h + h;
+        h += h;
     }
-    for i in 0..old.len() + 1 {
-        I[usz(V[i])] = i as isize;
+    for (i, v) in V[0..old.len() + 1].iter().copied().enumerate() {
+        I[usz(v)] = i as isize;
     }
 }
 
@@ -181,24 +180,24 @@ fn matchlen(old: &[u8], new: &[u8]) -> usize {
     old.iter().zip(new).take_while(|(a, b)| a == b).count()
 }
 
-fn search(I: &[isize], old: &[u8], new: &[u8], st: usize, en: usize) -> (isize, usize) {
-    if en < 2 + st {
-        let x = matchlen(&old[usz(I[st])..], new);
-        let y = matchlen(&old[usz(I[en])..], new);
+fn search(I: &[isize], old: &[u8], new: &[u8]) -> (isize, usize) {
+    if I.len() < 3 {
+        let x = matchlen(&old[usz(I[0])..], new);
+        let y = matchlen(&old[usz(I[I.len()-1])..], new);
         if x > y {
-            (I[st], x)
+            (I[0], x)
         } else {
-            (I[en], y)
+            (I[I.len()-1], y)
         }
     } else {
-        let x = usz(st as isize + (en as isize - st as isize) / 2);
-        let left = &old[usz(I[x])..];
+        let mid = (I.len()-1) / 2;
+        let left = &old[usz(I[mid])..];
         let right = new;
         let len_to_check = left.len().min(right.len());
         if left[..len_to_check] < right[..len_to_check] {
-            search(I, old, new, x, en)
+            search(&I[mid..], old, new)
         } else {
-            search(I, old, new, st, x)
+            search(&I[..=mid], old, new)
         }
     }
 }
@@ -215,7 +214,7 @@ fn offtout(x: isize, buf: &mut [u8]) {
 
 fn bsdiff_internal(old: &[u8], new: &[u8], writer: &mut dyn Write) -> io::Result<()> {
     let mut I = vec![0; old.len() + 1];
-    let mut buffer = vec![0; new.len() + 1];
+    let mut buffer = Vec::new();
     let mut V = vec![0; old.len() + 1];
     qsufsort(&mut I, &mut V, old);
 
@@ -230,7 +229,7 @@ fn bsdiff_internal(old: &[u8], new: &[u8], writer: &mut dyn Write) -> io::Result
         scan += len;
         let mut scsc = scan;
         while scan < new.len() {
-            let (p, l) = search(&I, old, &new[scan..], 0, old.len());
+            let (p, l) = search(&I[..=old.len()], old, &new[scan..]);
             pos = usz(p);
             len = l;
             while scsc < scan + len {
@@ -317,14 +316,18 @@ fn bsdiff_internal(old: &[u8], new: &[u8], writer: &mut dyn Write) -> io::Result
         );
         writer.write_all(&buf[..24])?;
 
-        for i in 0..lenf {
-            buffer[i] = new[lastscan + i].wrapping_sub(old[lastpos + i]);
-        }
-        writer.write_all(&buffer[..lenf])?;
-        for i in 0..scan - lenb - (lastscan + lenf) {
-            buffer[i] = new[lastscan + lenf + i];
-        }
-        writer.write_all(&buffer[..(scan - lenb - (lastscan + lenf))])?;
+        buffer.clear();
+        buffer.extend(
+            new[lastscan..lastscan + lenf]
+                .iter()
+                .zip(&old[lastpos..lastpos + lenf])
+                .map(|(n, o)| n.wrapping_sub(*o)),
+        );
+        writer.write_all(&buffer)?;
+
+        let write_len = scan - lenb - (lastscan + lenf);
+        let write_start = lastscan + lenf;
+        writer.write_all(&new[write_start..write_start + write_len])?;
 
         lastscan = scan - lenb;
         lastpos = pos - lenb;
